package com.sumancs.flutterpaytmpayment

import android.app.Activity
import android.os.Bundle
import android.util.Log
import androidx.annotation.NonNull
import com.paytm.pgsdk.PaytmOrder
import com.paytm.pgsdk.PaytmPGService
import com.paytm.pgsdk.PaytmPaymentTransactionCallback

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.util.*

/** FlutterpaytmpaymentPlugin */
public class FlutterpaytmpaymentPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  var TAG = javaClass.name
  private val activity: Activity? = null
  var result: Result? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "flutterpaytmpayment")
    channel.setMethodCallHandler(this);
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "flutterpaytmpayment")
      channel.setMethodCallHandler(FlutterpaytmpaymentPlugin())
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    this.result = result;

    if (call.method == "startPaytmPayment") {
      val testing: Boolean = call.argument("testing")!!
      val mId: String = call.argument("mId")!!
      val orderId: String = call.argument("orderId")!!
      val custId: String = call.argument("custId")!!
      val channelId: String = call.argument("channelId")!!
      val txnAmount: String = call.argument("txnAmount")!!
      val website: String = call.argument("website")!!
      val callBackUrl: String = call.argument("callBackUrl")!!
      val industryTypeId: String = call.argument("industryTypeId")!!
      val checkSumHash: String = call.argument("checkSumHash")!!

      initializePaytmPayment(testing, mId, orderId, custId, channelId, txnAmount, website, callBackUrl, industryTypeId, checkSumHash)
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  private fun sendResponse(paramMap: Map<String, Any>) {
    result?.success(paramMap)
  }

  private fun initializePaytmPayment(testing: Boolean, mId: String,
                                     orderId: String,
                                     custId: String,
                                     channelId: String,
                                     txnAmount: String,
                                     website: String,
                                     callBackUrl: String,
                                     industryTypeId: String,
                                     checkSumHash: String) {

    val Service: PaytmPGService
    Service = if (testing) {
      PaytmPGService.getStagingService()
    } else {
      PaytmPGService.getProductionService()
    }

    //creating a hashmap and adding all the values required
    val paramMap = HashMap<String, String>()
    paramMap["MID"] = mId
    paramMap["ORDER_ID"] = orderId
    paramMap["CUST_ID"] = custId
    paramMap["CHANNEL_ID"] = channelId
    paramMap["TXN_AMOUNT"] = txnAmount
    paramMap["WEBSITE"] = website
    paramMap["CALLBACK_URL"] = callBackUrl
    paramMap["CHECKSUMHASH"] = checkSumHash
    paramMap["INDUSTRY_TYPE_ID"] = industryTypeId
    //creating a paytm order object using the hashmap
    val order = PaytmOrder(paramMap)

    //intializing the paytm service
    Service.initialize(order, null)

    //finally starting the payment transaction
    Service.startPaymentTransaction(activity, true, true, object : PaytmPaymentTransactionCallback {
      override fun onTransactionResponse(bundle: Bundle) {
        Log.i(TAG, bundle.toString())
        val paramMap: MutableMap<String, Any> = HashMap()
        for (key in bundle.keySet()) {
          paramMap[key] = bundle.getString(key)
        }
        Log.i(TAG, paramMap.toString())
        sendResponse(paramMap)
      }

      override fun networkNotAvailable() {
        val paramMap: MutableMap<String, Any> = HashMap()
        paramMap["error"] = true
        paramMap["errorMessage"] = "Network Not Available"
        sendResponse(paramMap)
      }

      override fun clientAuthenticationFailed(s: String) {
        val paramMap: MutableMap<String, Any> = HashMap()
        paramMap["error"] = true
        paramMap["errorMessage"] = s
        sendResponse(paramMap)
      }

      override fun someUIErrorOccurred(s: String) {
        val paramMap: MutableMap<String, Any> = HashMap()
        paramMap["error"] = true
        paramMap["errorMessage"] = s
        sendResponse(paramMap)
      }

      override fun onErrorLoadingWebPage(i: Int, s: String, s1: String) {
        val paramMap: MutableMap<String, Any> = HashMap()
        paramMap["error"] = true
        paramMap["errorMessage"] = "$s , $s1"
        sendResponse(paramMap)
      }

      override fun onBackPressedCancelTransaction() {
        val paramMap: MutableMap<String, Any> = HashMap()
        paramMap["error"] = true
        paramMap["errorMessage"] = "Back Pressed Transaction Cancelled"
        sendResponse(paramMap)
      }

      override fun onTransactionCancel(s: String, bundle: Bundle) {
        Log.i(TAG, s + bundle.toString())
        val paramMap: MutableMap<String, Any> = HashMap()
        for (key in bundle.keySet()) {
          paramMap[key] = bundle.getString(key)
        }
        Log.i(TAG, paramMap.toString())
        paramMap["error"] = true
        paramMap["errorMessage"] = s
        sendResponse(paramMap)
      }
    })
  }
}
